@@ -4,8 +4,10 @@
 #'
 #'@param hillshade A three-dimensional RGB array or 2D matrix of shadow intensities. 
 #'@param shadowmap A matrix that incidates the intensity of the shadow at that point. 0 is full darkness, 1 is full light.
-#'@param max_darken Default 0.7. The lower limit for how much the image will be darkened. 0 is completely black,
+#'@param max_darken Default `0.7`. The lower limit for how much the image will be darkened. 0 is completely black,
 #'1 means the shadow map will have no effect.
+#'@param rescale_original Default `FALSE`. If `TRUE`, `hillshade` will be scaled to match the dimensions of `shadowmap` (instead of
+#'the other way around).
 #'@return Shaded texture map.
 #'@export
 #'@examples
@@ -40,15 +42,27 @@
 #'  add_shadow(ray_shade(montereybay,sunaltitude=20,zscale=50),max_darken=0.7) %>%
 #'  plot_map()
 #'}
-add_shadow = function(hillshade, shadowmap, max_darken = 0.7) {
+add_shadow = function(hillshade, shadowmap, max_darken = 0.7, rescale_original = FALSE) {
   if(length(dim(shadowmap)) == 3 && length(dim(hillshade)) == 2) {
     tempstore = hillshade
     hillshade = shadowmap
     shadowmap = tempstore
   }
-  shadowmap = t(flipud(shadowmap))
   if(length(dim(hillshade)) == 3) {
     hillshade = hillshade ^ 2.2
+    if(!all(dim(hillshade)[1:2] == dim(shadowmap))) {
+      if(rescale_original) {
+        temphillshade = array(0, dim = c(dim(shadowmap),3))
+        temphillshade[,,1] = rayimage::render_resized(hillshade[,,1], dims = dim(shadowmap))
+        temphillshade[,,2] = rayimage::render_resized(hillshade[,,2], dims = dim(shadowmap))
+        temphillshade[,,3] = rayimage::render_resized(hillshade[,,3], dims = dim(shadowmap))
+        hillshade = temphillshade
+      } else {
+        shadowmap = t(flipud(rayimage::render_resized(shadowmap, dims = dim(hillshade))))
+      }
+    } else {
+      shadowmap = t(flipud(shadowmap))
+    }
     hillshade[,,1] = hillshade[,,1] * scales::rescale(shadowmap,c(max_darken,1))
     hillshade[,,2] = hillshade[,,2] * scales::rescale(shadowmap,c(max_darken,1))
     hillshade[,,3] = hillshade[,,3] * scales::rescale(shadowmap,c(max_darken,1))
